.view {
	background-color: white;
}

.viewContainer {
	font-family: monospace;
	font-size: 10pt;
}

.viewContent {
}/* Styles for rulers */
.ruler {
	background-color: white;
}
.ruler.annotations {
	border-right: 1px solid lightgray;
	width: 16px;
}
.ruler.folding {
	border-right: 1px solid lightgray;
	width: 14px;
}
.ruler.lines {
	border-right: 1px solid lightgray;
	text-align: right;
}
.ruler.overview {
	width: 14px;
}

/* Styles for the line number ruler */
.rulerLines {
	background-color: white;
}
.rulerLines.even
.rulerLines.odd {
}.token_singleline_comment {
	color: green;
}

.token_multiline_comment {
	color: green;
}

.token_doc_comment {
	color: #00008F;
}

.token_doc_html_markup {
	color: #7F7F9F;
}

.token_doc_tag {
	color: #7F9FBF;
}

.token_task_tag {
	color: #7F9FBF;
}

.token_string {
	color: blue;
}

.token_keyword {
	color: darkred;
	font-weight: bold;
}

.token_bracket_outline {
	outline: 1px solid red;
}

.token_bracket {
	color: white;
	background-color: grey;
}

.token_space {
	/* images/white_space.png */
	background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAkAAAAJCAIAAABv85FHAAAABnRSTlMA/wAAAACkwsAdAAAAIUlEQVR4nGP4z8CAC+GUIEXuABhgkTuABEiRw2cmae4EAH05X7xDolNRAAAAAElFTkSuQmCC");
	background-repeat: no-repeat;
 	background-position: center center;
}

.token_tab {
	/* images/white_tab.png */
	background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAwAAAAJCAIAAACJ2loDAAAABnRSTlMA/wD/AP83WBt9AAAAMklEQVR4nGP4TwRgoK6i52c3bz5w6zMSA6tJn28d2Lx589nnCAYu63AaSLxJRLoJPwAAeNk0aG4opfMAAAAASUVORK5CYII=");
	background-repeat: no-repeat;
 	background-position: left center;
}

.line_caret {
	background-color: #EAF2FE;
}

/* Styling for html syntax highlighting */
.entity-name-tag {
	color: #3f7f7f;
}

.entity-other-attribute-name {
	color: #7f007f;
}

.punctuation-definition-comment {
	color: #3f5fbf;
}

.comment {
	color: #3f5fbf
}

.string-quoted {
	color: #2a00ff;
	font-style: italic;
}

.invalid {
	color: red;
	font-weight: bold;
}