#!/usr/bin/perl -w
#
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is ``syms-by-addr.sh''
#
# The Initial Developer of the Original Code is Netscape Communications Corp.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Chris Waterson <waterson@netscape.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

#
# List the missing symbols (or, do a set difference).
#

use 5.004;
use strict;
use Getopt::Long;

$::opt_help = 0;
$::opt_defined = "mozilla-bin.order";
GetOptions("help", "defined=s");

if ($::opt_help) {
    die "usage: missing-syms.pl --defined=<file> <symlist>";
}

$::Defined = { };

open(DEFINED, "<$::opt_defined") || die "couldn't open defined symbol list $::opt_defined";
while (<DEFINED>) {
    chomp;
    $::Defined{$_} = 1;
}
close(DEFINED);

while (<>) {
    chomp;
    print "$_\n" unless ($::Defined{$_});
}

